'use strict';

var _ = require('lodash');

var yaml = require('js-yaml')
var Colr = require('colr')
var createExporter = require('../exporter');

var template = "colors:\n  # Default colors\n  primary:\n    background: '0x{{=c.background}}'\n    foreground: '0x{{=c.foreground}}'\n\n  # Normal colors\n  normal:\n    black:   '0x{{=c[0]}}'\n    red:     '0x{{=c[1]}}'\n    green:   '0x{{=c[2]}}'\n    yellow:  '0x{{=c[3]}}'\n    blue:    '0x{{=c[4]}}'\n    magenta: '0x{{=c[5]}}'\n    cyan:    '0x{{=c[6]}}'\n    white:   '0x{{=c[7]}}'\n\n  # Bright colors\n  bright:\n    black:   '0x{{=c[8]}}'\n    red:     '0x{{=c[9]}}'\n    green:   '0x{{=c[10]}}'\n    yellow:  '0x{{=c[11]}}'\n    blue:    '0x{{=c[12]}}'\n    magenta: '0x{{=c[13]}}'\n    cyan:    '0x{{=c[14]}}'\n    white:   '0x{{=c[15]}}'\n";

function hex (string) {
  return Colr.fromHex(string.substring(2))
}

module.exports = {
  export: createExporter(template, _.partialRight(_.mapValues, function (color) {
    return color.toHex().slice(1);
  })),
  import: function (input) {
    var doc = yaml.safeLoad(input);
    var colors = doc.colors;
    var primary = colors.primary;
    var normal = colors.normal;
    var bright = colors.bright;

    return {
      background: hex(primary.background),
      foreground: hex(primary.foreground),
      0: hex(normal.black),
      1: hex(normal.red),
      2: hex(normal.green),
      3: hex(normal.yellow),
      4: hex(normal.blue),
      5: hex(normal.magenta),
      6: hex(normal.cyan),
      7: hex(normal.white),
      8: hex(bright.black),
      9: hex(bright.red),
      10: hex(bright.green),
      11: hex(bright.yellow),
      12: hex(bright.blue),
      13: hex(bright.magenta),
      14: hex(bright.cyan),
      15: hex(bright.white),
    }
  }
};
