'use strict';

var _ = require('lodash');

var Colr = require('colr');
var createExporter = require('../exporter');

var template = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE plist PUBLIC \"-//Apple//DTD PLIST 1.0//EN\" \"http://www.apple.com/DTDs/PropertyList-1.0.dtd\">\n<plist version=\"1.0\">\n<dict>\n\t<key>Ansi 0 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[0][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[0][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[0][0]}}</real>\n\t</dict>\n\t<key>Ansi 1 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[1][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[1][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[1][0]}}</real>\n\t</dict>\n\t<key>Ansi 10 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[10][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[10][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[10][0]}}</real>\n\t</dict>\n\t<key>Ansi 11 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[11][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[11][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[11][0]}}</real>\n\t</dict>\n\t<key>Ansi 12 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[12][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[12][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[12][0]}}</real>\n\t</dict>\n\t<key>Ansi 13 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[13][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[13][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[13][0]}}</real>\n\t</dict>\n\t<key>Ansi 14 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[14][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[14][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[14][0]}}</real>\n\t</dict>\n\t<key>Ansi 15 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[15][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[15][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[15][0]}}</real>\n\t</dict>\n\t<key>Ansi 2 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[2][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[2][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[2][0]}}</real>\n\t</dict>\n\t<key>Ansi 3 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[3][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[3][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[3][0]}}</real>\n\t</dict>\n\t<key>Ansi 4 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[4][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[4][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[4][0]}}</real>\n\t</dict>\n\t<key>Ansi 5 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[5][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[5][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[5][0]}}</real>\n\t</dict>\n\t<key>Ansi 6 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[6][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[6][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[6][0]}}</real>\n\t</dict>\n\t<key>Ansi 7 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[7][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[7][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[7][0]}}</real>\n\t</dict>\n\t<key>Ansi 8 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[8][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[8][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[8][0]}}</real>\n\t</dict>\n\t<key>Ansi 9 Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c[9][2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c[9][1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c[9][0]}}</real>\n\t</dict>\n\t<key>Background Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c.background[2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c.background[1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c.background[0]}}</real>\n\t</dict>\n\t<key>Bold Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c.foreground[2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c.foreground[1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c.foreground[0]}}</real>\n\t</dict>\n\t<key>Cursor Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c.foreground[2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c.foreground[1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c.foreground[0]}}</real>\n\t</dict>\n\t<key>Cursor Text Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c.background[2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c.background[1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c.background[0]}}</real>\n\t</dict>\n\t<key>Foreground Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c.foreground[2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c.foreground[1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c.foreground[0]}}</real>\n\t</dict>\n\t<key>Selected Text Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c.foreground[2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c.foreground[1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c.foreground[0]}}</real>\n\t</dict>\n\t<key>Selection Color</key>\n\t<dict>\n\t\t<key>Color Space</key>\n\t\t<string>sRGB</string>\n\t\t<key>Blue Component</key>\n\t\t<real>{{=c.background[2]}}</real>\n\t\t<key>Green Component</key>\n\t\t<real>{{=c.background[1]}}</real>\n\t\t<key>Red Component</key>\n\t\t<real>{{=c.background[0]}}</real>\n\t</dict>\n</dict>\n</plist>\n";

var lookup = {
  'Background Color': 'background',
  'Foreground Color': 'foreground',
  'Ansi 0 Color': '0',
  'Ansi 8 Color': '8',
  'Ansi 1 Color': '1',
  'Ansi 9 Color': '9',
  'Ansi 2 Color': '2',
  'Ansi 10 Color': '10',
  'Ansi 3 Color': '3',
  'Ansi 11 Color': '11',
  'Ansi 4 Color': '4',
  'Ansi 12 Color': '12',
  'Ansi 5 Color': '5',
  'Ansi 13 Color': '13',
  'Ansi 6 Color': '6',
  'Ansi 14 Color': '14',
  'Ansi 7 Color': '7',
  'Ansi 15 Color': '15'
};

var regex = {
  group: /<key>([^<]*)<\/key>\s*<dict>/gi,
  component: /<key>(Blue|Green|Red) Component<\/key>/gi,
  real: /<real>([\d.]+)<\/real>/gi
};

module.exports = {

  import: function (input) {
    regex.group.lastIndex = 0;
    var output = {};

    var group;
    while ((group = regex.group.exec(input)) !== null) {
      var groupName = group[1];
      var colorName = lookup[groupName];
      if (! colorName) { continue; }

      regex.component.lastIndex = group.index;
      var color = {};
      for (var i = 0; i < 3; i += 1) {
        var component = regex.component.exec(input);
        if (! component) { continue; }

        var componentName = component[1].toLowerCase();
        regex.real.lastIndex = component.index;
        color[componentName] = regex.real.exec(input)[1] * 255;
      }
      output[colorName] = Colr.fromRgb(color.red, color.green, color.blue);
    }

    return output;
  },

  export: createExporter(template, _.partialRight(_.mapValues, function (color) {
    return color.toAvgRgbArray();
  }))

};
