'use strict';

var _ = require('lodash');

var Colr = require('colr');
var createExporter = require('../exporter');

var template = "# special\nactive_border_color     {{=c.foreground}}\ninactive_border_color   {{=c.background}}\nbell_border_color       {{=c[3]}}\nurl_color               {{=c[2]}}\n\nactive_tab_foreground   {{=c.foreground}}\nactive_tab_background   {{=c.background}}\ninactive_tab_foreground {{=c[8]}}\ninactive_tab_background {{=c.background}}\n\ncursor                  {{=c.foreground}}\nforeground              {{=c.foreground}}\nbackground              {{=c.background}}\nselection_foreground    {{=c[15]}}\nselection_background    {{=c[0]}}\n\n# black\ncolor0                  {{=c[0]}}\ncolor8                  {{=c[8]}}\n\n# red\ncolor1                  {{=c[1]}}\ncolor9                  {{=c[9]}}\n\n# green\ncolor2                  {{=c[2]}}\ncolor10                 {{=c[10]}}\n\n# yellow\ncolor3                  {{=c[3]}}\ncolor11                 {{=c[11]}}\n\n# blue\ncolor4                  {{=c[4]}}\ncolor12                 {{=c[12]}}\n\n# magenta\ncolor5                  {{=c[5]}}\ncolor13                 {{=c[13]}}\n\n# cyan\ncolor6                  {{=c[6]}}\ncolor14                 {{=c[14]}}\n\n# white\ncolor7                  {{=c[7]}}\ncolor15                 {{=c[15]}}\n";

/*
 * Regexes
 */

var hex = '(?:\\s*)' + // space
    '(#[a-f0-9]{6})';  // hex

var regex = {
  color: new RegExp('^(foreground|background|color(\\d{1,2}))'+hex+'$', 'mig'),
  comment: /^\s*#.*$/mg
};

var VALID_COLORS = [
  '0', '1', '2', '3', '4', '5', '6', '7',
  '8', '9', '10', '11', '12', '13', '14', '15',
  'foreground', 'background'
];

module.exports = {
  export: createExporter(template, _.partialRight(_.mapValues, function (color) {
    return color.toHex();
  })),


  /*
   * kitty.import
   *
   * - input (string) : text to parse
   * > colors (object)
   */

  import: function (input) {
    var output = {};
    var match, search;

    // remove comments
    input = input.replace(regex.comment, '');

    // match colors
    while ((match = regex.color.exec(input)) !== null) {
      // if is colorN use N else use foreground/background
      var index = match[2] ? match[2] : match[1];
      var value = match[3];

      //ignore invalid color names
      if (VALID_COLORS.indexOf(index) < 0) { continue; }

      output[index] = Colr.fromHex(value);
    }

    return output;
  }
};
