'use strict';

var _ = require('lodash');

var Colr = require('colr');
var createExporter = require('../exporter');

var template = "[colors]\n\n# special\nforeground      = {{=c.foreground}}\nforeground_bold = {{=c.foreground}}\ncursor          = {{=c.foreground}}\nbackground      = {{=c.background}}\n\n# black\ncolor0  = {{=c[0]}}\ncolor8  = {{=c[8]}}\n\n# red\ncolor1  = {{=c[1]}}\ncolor9  = {{=c[9]}}\n\n# green\ncolor2  = {{=c[2]}}\ncolor10 = {{=c[10]}}\n\n# yellow\ncolor3  = {{=c[3]}}\ncolor11 = {{=c[11]}}\n\n# blue\ncolor4  = {{=c[4]}}\ncolor12 = {{=c[12]}}\n\n# magenta\ncolor5  = {{=c[5]}}\ncolor13 = {{=c[13]}}\n\n# cyan\ncolor6  = {{=c[6]}}\ncolor14 = {{=c[14]}}\n\n# white\ncolor7  = {{=c[7]}}\ncolor15 = {{=c[15]}}\n";

var regex = {
  color: /(foreground|background|color(\d\d?))\s*=\s*(#[a-f0-9]{6})/gi,
  comment: /^\s*#.*$/mg
};

module.exports = {

  import: function (input) {
    var output = {};
    var match;

    // remove comments
    input = input.replace(regex.comment, '');

    // match colors
    regex.color.lastIndex = 0;
    while ((match = regex.color.exec(input)) !== null) {
      // if is colorN use N else use foreground/background
      var index = match[2] ? match[2] : match[1];
      output[index] = Colr.fromHex(match[3]);
    }

    return output;
  },

  export: createExporter(template, _.partialRight(_.mapValues, function (color) {
    return color.toHex();
  }))

};
