'use strict';

var _ = require('lodash');

var Colr = require('colr');
var createExporter = require('../exporter');

var template = "{\n    \"wallpaper\": \"\",\n    \"alpha\": \"100\",\n    \"special\": {\n        \"background\": \"{{=c.background}}\",\n        \"foreground\": \"{{=c.foreground}}\",\n        \"cursor\": \"{{=c.foreground}}\"\n    },\n    \"colors\": {\n        \"color0\": \"{{=c[0]}}\",\n        \"color1\": \"{{=c[1]}}\",\n        \"color2\": \"{{=c[2]}}\",\n        \"color3\": \"{{=c[3]}}\",\n        \"color4\": \"{{=c[4]}}\",\n        \"color5\": \"{{=c[5]}}\",\n        \"color6\": \"{{=c[6]}}\",\n        \"color7\": \"{{=c[7]}}\",\n        \"color8\": \"{{=c[8]}}\",\n        \"color9\": \"{{=c[9]}}\",\n        \"color10\": \"{{=c[10]}}\",\n        \"color11\": \"{{=c[11]}}\",\n        \"color12\": \"{{=c[12]}}\",\n        \"color13\": \"{{=c[13]}}\",\n        \"color14\": \"{{=c[14]}}\",\n        \"color15\": \"{{=c[15]}}\"\n    }\n}\n";

/*
 * Regexes
 */

var hex = '(?:\\s*)' + // space
    '(#[a-f0-9]{6})';  // hex

var regex = {
  color: new RegExp('^(foreground|background|color(\\d{1,2}))'+hex+'$', 'mig'),
  comment: /^\s*#.*$/mg
};

var VALID_COLORS = [
  '0', '1', '2', '3', '4', '5', '6', '7',
  '8', '9', '10', '11', '12', '13', '14', '15',
  'foreground', 'background'
];

module.exports = {
  export: createExporter(template, _.partialRight(_.mapValues, function (color) {
    return color.toHex();
  })),


  /*
   * wal.import
   *
   * - input (string) : text to parse
   * > colors (object)
   */

  import: function (input) {
    var output = {};
    var match, search;

    // remove comments
    input = input.replace(regex.comment, '');

    // match colors
    while ((match = regex.color.exec(input)) !== null) {
      // if is colorN use N else use foreground/background
      var index = match[2] ? match[2] : match[1];
      var value = match[3];

      //ignore invalid color names
      if (VALID_COLORS.indexOf(index) < 0) { continue; }

      output[index] = Colr.fromHex(value);
    }

    return output;
  }
};

