'use strict';

var _ = require('lodash');

var createExporter = require('../exporter');

var template = "#!/usr/bin/env bash\n# Base16 - Gnome Terminal color scheme install script\n\n[[ -z \"$PROFILE_NAME\" ]] && PROFILE_NAME=\"terminal.sexy\"\n[[ -z \"$PROFILE_SLUG\" ]] && PROFILE_SLUG=\"terminal-dot-sexy\"\n[[ -z \"$DCONF\" ]] && DCONF=dconf\n[[ -z \"$UUIDGEN\" ]] && UUIDGEN=uuidgen\n\ndset() {\n    local key=\"$1\"; shift\n    local val=\"$1\"; shift\n\n    if [[ \"$type\" == \"string\" ]]; then\n        val=\"'$val'\"\n    fi\n\n    \"$DCONF\" write \"$PROFILE_KEY/$key\" \"$val\"\n}\n\n# because dconf still doesn't have \"append\"\ndlist_append() {\n    local key=\"$1\"; shift\n    local val=\"$1\"; shift\n\n    local entries=\"$(\n        {\n            \"$DCONF\" read \"$key\" | tr -d '[]' | tr , \"\\n\" | fgrep -v \"$val\"\n            echo \"'$val'\"\n        } | head -c-1 | tr \"\\n\" ,\n    )\"\n\n    \"$DCONF\" write \"$key\" \"[$entries]\"\n}\n\n# Newest versions of gnome-terminal use dconf\nif which \"$DCONF\" > /dev/null 2>&1; then\n    [[ -z \"$BASE_KEY_NEW\" ]] && BASE_KEY_NEW=/org/gnome/terminal/legacy/profiles:\n\n    if [[ -n \"`$DCONF list $BASE_KEY_NEW/`\" ]]; then\n        if which \"$UUIDGEN\" > /dev/null 2>&1; then\n            PROFILE_SLUG=`uuidgen`\n        fi\n\n        if [[ -n \"`$DCONF read $BASE_KEY_NEW/default`\" ]]; then\n            DEFAULT_SLUG=`$DCONF read $BASE_KEY_NEW/default | tr -d \\'`\n        else\n            DEFAULT_SLUG=`$DCONF list $BASE_KEY_NEW/ | grep '^:' | head -n1 | tr -d :/`\n        fi\n\n        DEFAULT_KEY=\"$BASE_KEY_NEW/:$DEFAULT_SLUG\"\n        PROFILE_KEY=\"$BASE_KEY_NEW/:$PROFILE_SLUG\"\n\n        # copy existing settings from default profile\n        $DCONF dump \"$DEFAULT_KEY/\" | $DCONF load \"$PROFILE_KEY/\"\n\n        # add new copy to list of profiles\n        dlist_append $BASE_KEY_NEW/list \"$PROFILE_SLUG\"\n\n        # update profile values with theme options\n        dset visible-name \"'$PROFILE_NAME'\"\n        dset palette \"['{{=c[0]}}', '{{=c[1]}}', '{{=c[2]}}', '{{=c[3]}}', '{{=c[4]}}', '{{=c[5]}}', '{{=c[6]}}', '{{=c[7]}}', '{{=c[8]}}', '{{=c[9]}}', '{{=c[10]}}', '{{=c[11]}}', '{{=c[12]}}', '{{=c[13]}}', '{{=c[14]}}', '{{=c[15]}}']\"\n        dset background-color \"'{{=c.background}}'\"\n        dset foreground-color \"'{{=c.foreground}}'\"\n        dset bold-color \"'{{=c.foreground}}'\"\n        dset bold-color-same-as-fg \"true\"\n        dset use-theme-colors \"false\"\n        dset use-theme-background \"false\"\n\n        unset PROFILE_NAME\n        unset PROFILE_SLUG\n        unset DCONF\n        unset UUIDGEN\n        exit 0\n    fi\nfi\n\n# Fallback for Gnome 2 and early Gnome 3\n[[ -z \"$GCONFTOOL\" ]] && GCONFTOOL=gconftool\n[[ -z \"$BASE_KEY\" ]] && BASE_KEY=/apps/gnome-terminal/profiles\n\nPROFILE_KEY=\"$BASE_KEY/$PROFILE_SLUG\"\n\ngset() {\n    local type=\"$1\"; shift\n    local key=\"$1\"; shift\n    local val=\"$1\"; shift\n\n    \"$GCONFTOOL\" --set --type \"$type\" \"$PROFILE_KEY/$key\" -- \"$val\"\n}\n\n# Because gconftool doesn't have \"append\"\nglist_append() {\n    local type=\"$1\"; shift\n    local key=\"$1\"; shift\n    local val=\"$1\"; shift\n\n    local entries=\"$(\n        {\n            \"$GCONFTOOL\" --get \"$key\" | tr -d '[]' | tr , \"\\n\" | fgrep -v \"$val\"\n            echo \"$val\"\n        } | head -c-1 | tr \"\\n\" ,\n    )\"\n\n    \"$GCONFTOOL\" --set --type list --list-type $type \"$key\" \"[$entries]\"\n}\n\n# Append the Base16 profile to the profile list\nglist_append string /apps/gnome-terminal/global/profile_list \"$PROFILE_SLUG\"\n\ngset string visible_name \"$PROFILE_NAME\"\ngset string palette \"{{=c[0]}}:{{=c[1]}}:{{=c[2]}}:{{=c[3]}}:{{=c[4]}}:{{=c[5]}}:{{=c[6]}}:{{=c[7]}}:{{=c[8]}}:{{=c[9]}}:{{=c[10]}}:{{=c[11]}}:{{=c[12]}}:{{=c[13]}}:{{=c[14]}}:{{=c[15]}}\"\ngset string background_color \"{{=c.background}}\"\ngset string foreground_color \"{{=c.foreground}}\"\ngset string bold_color \"{{=c.foreground}}\"\ngset bool   bold_color_same_as_fg \"true\"\ngset bool   use_theme_colors \"false\"\ngset bool   use_theme_background \"false\"\n\nunset PROFILE_NAME\nunset PROFILE_SLUG\nunset DCONF\nunset UUIDGEN\n";

module.exports = {
  export: createExporter(template, _.partialRight(_.mapValues, function (color) {
    return color.toHex();
  }))
};
